﻿using CallCenter.Caches;
using CallCenter.Devices;
using CallCenter.Share.Enums;
using CallCenter.Tels;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Options;
using NewRock.Sdk;
using NewRock.Sdk.Control.Request;
using NewRock.Sdk.Control.Response;
using NewRock.Sdk.Manage.Request;
using NewRock.Sdk.Transfer.Conference.Request;
using NewRock.Sdk.Transfer.Connect.Request;
using NewRock.Sdk.Transfer.Queue.Request;
using System.Diagnostics;

using Ext = NewRock.Sdk.Control.Request.Base.Ext;
using Group = NewRock.Sdk.Control.Request.Group;
using VisitorToExtVisitor = NewRock.Sdk.Transfer.Connect.Request.VisitorToExtVisitor;

namespace CallCenter.Api.Controllers
{
    /// <summary>
    /// 设备测试专用，外部禁止调用
    /// </summary>
    public class TestSdkController : BaseController
    {
        private readonly INewRockClient _client;
        private readonly ILogger<TestController> _logger;
        private readonly IOptionsSnapshot<DeviceConfigs> _options;
        private readonly ITelCacheManager _telCacheManager;
        private readonly ITelGroupRepository _telGroupRepository;
        private readonly ITelRestRepository _telRestRepository;
        private readonly IUserCacheManager _userCacheManager;

        public TestSdkController(
            INewRockClient client,
            ILogger<TestController> logger,
            IOptionsSnapshot<DeviceConfigs> options,
            ITelCacheManager telCacheManager,
            ITelGroupRepository  telGroupRepository,
            ITelRestRepository telRestRepository,
            IUserCacheManager userCacheManager)
        {
            _client = client;
            _logger = logger;
            _options = options;
            _telCacheManager = telCacheManager;
            _telGroupRepository = telGroupRepository;
            _telRestRepository = telRestRepository;
            _userCacheManager = userCacheManager;
        }

        #region 测试加入分机组

        /// <summary>
        /// 将分机加入到分机组
        /// </summary>
        /// <returns></returns>
        [HttpPost("add-exttogroup")]
        public async Task AddExtToGroup(string extid)
        {
            Stopwatch stopwatch = new Stopwatch();
            var tel = _telCacheManager.GetTel(extid);
            foreach (var group in tel.Groups)
            {
                stopwatch.Start();
                var groupModel = new Group()
                {
                    Id = group.No,
                    Ext = new List<string>() { extid }
                };

                switch (group.Distribution)
                {
                    case EDistribution.Sequential:
                        groupModel.Distribution = "sequential";
                        break;
                    case EDistribution.Group:
                        groupModel.Distribution = "group";
                        break;
                    case EDistribution.Circular:
                        groupModel.Distribution = "circular";
                        break;
                    default:
                        break;
                }

                if (!string.IsNullOrEmpty(group.Voice))
                    groupModel.Voicefile = group.Voice;

                stopwatch.Stop();
                _logger.LogInformation("组装数据，分机组号：" + group.No + "，耗时：" + stopwatch.ElapsedMilliseconds + "ms");

                stopwatch.Restart();

                var resp = await _client.ConfigExtGroup(new AssginConfigGroupRequest()
                {
                    Attribute = "Assign",
                    Group = groupModel
                }, _options.Value.ReceiveKey,
                    _options.Value.Expired,
                    HttpContext.RequestAborted);
                stopwatch.Stop();
                _logger.LogInformation("更新设备，分机组号：" + group.No + "，耗时：" + stopwatch.ElapsedMilliseconds + "ms");
            }
        }

        /// <summary>
        /// 将分机移除分机组
        /// </summary>
        /// <returns></returns>
        [HttpPost("remove-extfromgroup")]
        public async Task RemoveExtFromGroup(string extid)
        {
            Stopwatch stopwatch = new Stopwatch();
            var tel = _telCacheManager.GetTel(extid);
            foreach (var group in tel.Groups)
            {
                

                stopwatch.Start();

                #region 清除分机组设置

                await _client.ConfigExtGroup(
                    new AssginConfigGroupRequest() { Attribute = "Assign", Group = new Group() { Id = group.No }, },
                    _options.Value.ReceiveKey,
                    _options.Value.Expired,
                    HttpContext.RequestAborted
                    );

                #endregion

                stopwatch.Stop();
                _logger.LogInformation("分机组清空，分机组：" + group.No + ",执行时间：" + stopwatch.ElapsedMilliseconds + "ms");


                stopwatch.Restart();
                var list = await _telGroupRepository.QueryExtAsync(d => d.No == group.No, d => d.Includes(x => x.Tels));

                //检查是否上班
                List<string> exts = new List<string>();
                foreach (var ext in list[0].Tels)
                {
                    var iswork = await _userCacheManager.IsWorkingByTelAsync(ext.No, HttpContext.RequestAborted);
                    if (iswork)
                    {
                        var teltemp = _telCacheManager.GetTel(ext.No);
                        if (teltemp.TelStatus == Tels.ETelStatus.Ready)
                        {
                            exts.Add(ext.No);
                        }
                    }
                }

                //查询所有正在休息的分机
                List<string> restexts = (await _telRestRepository.QueryAsync(x => x.EndTime == null)).Select(x => x.TelNo).ToList();
                if (restexts != null && restexts.Count > 0)
                {
                    foreach (var item in restexts)
                    {
                        exts.Remove(item);
                    }
                }

                //更新
                var groupModel = new Group()
                {
                    Id = group.No,
                    Voicefile = group.Voice
                };
                switch (group.Distribution)
                {
                    case EDistribution.Sequential:
                        groupModel.Distribution = "sequential";
                        break;
                    case EDistribution.Group:
                        groupModel.Distribution = "group";
                        break;
                    case EDistribution.Circular:
                        groupModel.Distribution = "circular";
                        break;
                    default:
                        break;
                }

                groupModel.Ext = exts;

                stopwatch.Stop();
                _logger.LogInformation("分机组组装数据，分机组：" + group.No + ",执行时间：" + stopwatch.ElapsedMilliseconds + "ms");

                stopwatch.Restart();
                await _client.ConfigExtGroup(
                    new AssginConfigGroupRequest() { Attribute = "Assign", Group = groupModel, },
                    _options.Value.ReceiveKey,
                    _options.Value.Expired,
                    HttpContext.RequestAborted
                    );
                stopwatch.Stop();
                _logger.LogInformation("更新设备分机组数据，分机组：" + group.No + "，执行时间：" + stopwatch.ElapsedMilliseconds + "ms");

            }
        }

        #endregion

        #region  查询（Query）

        /// <summary>
        /// 查询设备
        /// </summary>
        /// <returns></returns>
        [HttpGet("QueryDeviceInfo")]
        public async Task<QueryDeviceInfoResponse?> QueryDeviceInfo()
        {
            var result = await _client.QueryDeviceInfo(
                new QueryDeviceInfoRequest { Attribute = "Query", DeviceInfo = string.Empty },
                _options.Value.ReceiveKey,
                _options.Value.Expired,
                HttpContext.RequestAborted);
            _logger.LogInformation(System.Text.Json.JsonSerializer.Serialize(result));
            return result;
        }


        /// <summary>
        /// 查询分机
        /// </summary>
        /// <returns></returns>
        [HttpPost("QueryExt")]
        public async Task<QueryExtResponse?> QueryExt(string extid)
        {
            var result = await _client.QueryExt(
                new QueryExtRequest() { Attribute = "Query", Ext = new Ext() { Id = extid } },
                _options.Value.ReceiveKey,
                _options.Value.Expired,
                HttpContext.RequestAborted
            );
            _logger.LogInformation(System.Text.Json.JsonSerializer.Serialize(result));
            return result;
        }

        /// <summary>
        /// 查询分机组
        /// </summary>
        /// <param name="groupid"></param>
        /// <returns></returns>
        [HttpPost("QueryExtGroup")]
        public async Task<QueryExtGroupResponse?> QueryExtGroup(string? groupid)
        {
            var result = await _client.QueryExtGroup(new QueryExtGroupRequest()
            {
                Attribute = "Query",
                Group = new QueryExtGroup() { Id = groupid }
            }, _options.Value.ReceiveKey,
                _options.Value.Expired,
                HttpContext.RequestAborted);
            _logger.LogInformation(System.Text.Json.JsonSerializer.Serialize(result));
            return result;
        }

        /// <summary>
        /// 查询语音菜单
        /// </summary>
        /// <param name="menuid"></param>
        /// <returns></returns>
        [HttpPost("QueryMenu")]
        public async Task QueryMenu(string menuid)
        {
            var result = await _client.QueryMenu(new QueryMenuRequest()
            {
                Attribute = "Query",
                Menu = new QueryMenuMenu() { Id = menuid }
            }, _options.Value.ReceiveKey,
                _options.Value.Expired,
                HttpContext.RequestAborted);
            _logger.LogInformation(System.Text.Json.JsonSerializer.Serialize(result));
        }

        /// <summary>
        /// 查询中继
        /// </summary>
        /// <param name="trunkid"></param>
        /// <returns></returns>
        [HttpPost("QueryTrunk")]

        public async Task QueryTrunk(string trunkid)
        {
            var result = await _client.QueryTrunk(new QueryTrunkRequest()
            {
                Attribute = "Query",
                Trunk = new QueryTrunkTrunk() { Id = trunkid }
            },
                _options.Value.ReceiveKey,
                _options.Value.Expired,
                HttpContext.RequestAborted);
            _logger.LogInformation(System.Text.Json.JsonSerializer.Serialize(result));
        }

        /// <summary>
        /// 查询来电
        /// </summary>
        /// <param name="visitorid"></param>
        /// <returns></returns>
        [HttpPost("QueryVisitor")]
        public async Task QueryVisitor(string visitorid)
        {
            var result = await _client.QueryVisitor(new QueryVisitorRequest()
            {
                Attribute = "Query",
                Visitor = new QueryVisitorVisitor() { Id = visitorid }
            }, _options.Value.ReceiveKey,
                _options.Value.Expired,
                HttpContext.RequestAborted);
            _logger.LogInformation(System.Text.Json.JsonSerializer.Serialize(result));
        }

        /// <summary>
        /// 查询去电
        /// </summary>
        /// <param name="outerid"></param>
        /// <returns></returns>
        [HttpPost("QueryOuter")]
        public async Task QueryOuter(string outerid)
        {
            var result = await _client.QueryOuter(new QueryOuterRequest()
            {
                Attribute = "Query",
                Outer = new QueryOuterOuter() { Id = outerid }
            },
                _options.Value.ReceiveKey,
                _options.Value.Expired,
                HttpContext.RequestAborted);
            _logger.LogInformation(System.Text.Json.JsonSerializer.Serialize(result));
        }

        #endregion

        #region 强拆

        /// <summary>
        /// 强拆分机
        /// </summary>
        /// <param name="extid"></param>
        /// <returns></returns>
        [HttpPost("ClearExt")]
        public async Task ClearExt(string extid)
        {
            var result = await _client.ClearCall(new ClearCallRequest()
            {
                Attribute = "Clear",
                Ext = new Ext()
                {
                    Id = extid
                },
            }, _options.Value.ReceiveKey,
                _options.Value.Expired,
                HttpContext.RequestAborted);
        }

        /// <summary>
        /// 强拆来电
        /// </summary>
        /// <param name="visitorId"></param>
        /// <returns></returns>
        [HttpPost("ClearVisitor")]
        public async Task ClearVisitor(string visitorId)
        {
            var result = await _client.ClearCall(new ClearCallRequest()
            {
                Attribute = "Clear",
                Visitor = new ClearCallVisitor()
                {
                    Id = visitorId
                }
            }, _options.Value.ReceiveKey,
                _options.Value.Expired,
                HttpContext.RequestAborted);
        }

        /// <summary>
        /// 强拆去电
        /// </summary>
        /// <param name="outerId"></param>
        /// <returns></returns>
        [HttpPost("ClearOuter")]
        public async Task ClearOuter(string outerId)
        {
            var result = await _client.ClearCall(new ClearCallRequest()
            {
                Attribute = "Clear",
                Outer = new ClearCallOuter()
                {
                    Id = outerId
                }
            }, _options.Value.ReceiveKey,
                _options.Value.Expired,
                HttpContext.RequestAborted);
        }

        #endregion

        #region 配置（Assign）

        /// <summary>
        /// 配置分机
        /// </summary>
        /// <returns></returns>
        [HttpGet("ConfigExt")]
        public async Task ConfigExt()
        {
            var result = await _client.ConfigExt(new AssginConfigExtRequest()
            {
                Attribute = "Assign",
                Ext = new ConfigExt()
                {
                    Lineid = "IPPhone 14",
                    Id = "8013",
                    //Staffid = "038",
                    Groups = new List<string>()
                    {
                        "1",
                        "2",
                    },
                    //VoiceFile = "user_tts164815.dat",
                    //Call_Restriction = "2",
                    //Call_Pickup ="no",
                    No_Disturb = "on",
                    Fwd_Type = "0",
                    //Record="on",
                    Api = "7"
                }
            },
            _options.Value.ReceiveKey,
            _options.Value.Expired,
            HttpContext.RequestAborted);
            _logger.LogInformation(System.Text.Json.JsonSerializer.Serialize(result));
        }


        /// <summary>
        /// 配置分机组
        /// </summary>
        /// <returns></returns>
        [HttpGet("ConfigExtGroup")]
        public async Task ConfigExtGroup()
        {
            var result = await _client.ConfigExtGroup(new AssginConfigGroupRequest()
            {
                Attribute = "Assign",
                Group = new Group()
                {
                    Id = "1",
                    Voicefile = "NewMorning",
                    Distribution = "sequential",
                    Ext = new List<string>()
                    {
                        "8016",
                    }
                }
            }, _options.Value.ReceiveKey,
            _options.Value.Expired,
            HttpContext.RequestAborted);
            _logger.LogInformation(System.Text.Json.JsonSerializer.Serialize(result));
        }

        /// <summary>
        /// 配置语音菜单
        /// </summary>
        /// <returns></returns>
        [HttpGet("ConfigMenu")]
        public async Task ConfigMenu()
        {
            var result = await _client.ConfigMenu(new AssginConfigMenuRequest()
            {
                Attribute = "Assign",
                Menu = new AssginConfigMenuMenu()
                {
                    Id = "1",
                    VoiceFile = "user_tts131742",
                    Exit = "#",
                    Repeat = "3",
                    InfoLength = "5",
                }
            }, _options.Value.ReceiveKey,
                _options.Value.Expired,
                HttpContext.RequestAborted);
        }

        #endregion

        #region 呼叫保持和接回

        /// <summary>
        /// 呼叫保持
        /// </summary>
        /// <returns></returns>
        [HttpPost("Hold")]
        public async Task Hold(string extid)
        {
            var result = await _client.HoldOrUnHold(new HoldSetRequest()
            {
                Attribute = "Hold",
                Ext = new Ext() { Id = extid }
            }, _options.Value.ReceiveKey,
                _options.Value.Expired,
                HttpContext.RequestAborted);
            //_logger.LogInformation(result.Manufacturer);
        }

        /// <summary>
        /// 呼叫接回
        /// </summary>
        /// <returns></returns>
        [HttpPost("UnHold")]
        public async Task UnHold(string extid)
        {
            var result = await _client.HoldOrUnHold(new HoldSetRequest()
            {
                Attribute = "Unhold",
                Ext = new Ext() { Id = extid }
            }, _options.Value.ReceiveKey,
                _options.Value.Expired,
                HttpContext.RequestAborted);
        }

        #endregion

        #region 静音开启与解除

        /// <summary>
        /// 静音
        /// </summary>
        /// <param name="extid"></param>
        /// <returns></returns>
        [HttpPost("Mute")]
        public async Task Mute(string extid)
        {
            var result = await _client.MuteOrUnMute(new MuteSetRequest
            {
                Attribute = "Mute",
                Ext = new Ext() { Id = extid }
            }, _options.Value.ReceiveKey,
                _options.Value.Expired,
                HttpContext.RequestAborted);
        }

        /// <summary>
        /// 解除静音
        /// </summary>
        /// <param name="extid"></param>
        /// <returns></returns>
        [HttpPost("UnMute")]
        public async Task UnMute(string extid)
        {
            var result = await _client.MuteOrUnMute(new MuteSetRequest
            {
                Attribute = "Unmute",
                Ext = new Ext() { Id = extid }
            }, _options.Value.ReceiveKey,
                _options.Value.Expired,
                HttpContext.RequestAborted);
        }

        #endregion

        #region 呼叫转接命令（Transfer）

        #region 连接（Connect）

        /// <summary>
        /// 分机呼分机
        /// </summary>
        /// <returns></returns>
        [HttpPost("ExtensionToExtension")]
        public async Task ExtensionToExtension(string fromextid, string toextid)
        {
            var result = await _client.ExtensionToExtension(new ExtensionToExtensionRequest()
            {
                Attribute = "Connect",
                Exts = new List<ExtToExtExt>()
                    {
                        new ExtToExtExt() { Id = fromextid },
                        new ExtToExtExt() { Id = toextid }
                    }
            }, _options.Value.ReceiveKey,
                _options.Value.Expired,
                HttpContext.RequestAborted);
            _logger.LogInformation(System.Text.Json.JsonSerializer.Serialize(result));
        }

        /// <summary>
        /// 分机呼外部电话
        /// </summary>
        /// <returns></returns>
        [HttpPost("ExtToOuter")]
        public async Task ExtToOuter(string fromextid, string to)
        {
            var result = await _client.ExtToOuter(new ExtToOuterRequest()
            {
                Attribute = "Connect",
                Ext = new ExtToOuterExtRequest() { Id = fromextid },
                Outer = new ExtToOuterOuterRequest() { To = to }
            },
                _options.Value.ReceiveKey,
                _options.Value.Expired,
                HttpContext.RequestAborted);
        }

        /// <summary>
        /// 来电转分机
        /// </summary>
        /// <returns></returns>
        [HttpPost("VisitorToExt")]
        public async Task VisitorToExt(string visid, string toextid)
        {
            var result = await _client.VisitorToExt(new VisitorToExtRequest()
            {
                Attribute = "Connect",
                Visitor = new VisitorToExtVisitor() { Id = visid },
                Ext = new VisitorToExtExt() { Id = toextid }
            }, _options.Value.ReceiveKey,
                _options.Value.Expired,
                HttpContext.RequestAborted);
        }

        /// <summary>
        /// 来电转外部电话
        /// </summary>
        /// <param name="visid"></param>
        /// <param name="outerphonenum"></param>
        /// <param name="display"></param>
        /// <returns></returns>
        [HttpPost("VisitorToOuter")]
        public async Task VisitorToOuter(string visid, string outerphonenum, string display = "")
        {
            var result = await _client.VisitorToOuter(new VisitorToOuterRequest()
            {
                Attribute = "Connect",
                Visitor = new VisitorToOuterVisitor() { Id = visid },
                Outer = new VisitorToOuterOuter() { To = outerphonenum, Display = display },

            }, _options.Value.ReceiveKey,
                _options.Value.Expired,
                HttpContext.RequestAborted);
        }

        /// <summary>
        /// 来电转语音菜单
        /// </summary>
        /// <param name="visid"></param>
        /// <param name="menuid"></param>
        /// <returns></returns>
        [HttpPost("VisitorToMenu")]
        public async Task VisitorToMenu(string visid, string menuid)
        {
            var result = await _client.VisitorToMenu(new VisitorToMenuRequest()
            {
                Attribute = "Connect",
                Visitor = new VisitorToMenuVisitor() { Id = visid },
                Menu = new VisitorToMenuMenu() { Id = menuid }
            }, _options.Value.ReceiveKey,
                _options.Value.Expired,
                HttpContext.RequestAborted);
        }

        /// <summary>
        /// 去电转分机
        /// </summary>
        /// <param name="outer"></param>
        /// <param name="extid"></param>
        /// <returns></returns>
        [HttpPost("OuterToExt")]
        public async Task OuterToExt(string outer, string extid)
        {
            var result = await _client.OuterToExt(new OuterToExtRequest()
            {
                Attribute = "Connect",
                Outer = new OuterToExtOuter() { Id = outer },
                Ext = new OuterToExtExt() { Id = extid }
            },
                _options.Value.ReceiveKey,
                _options.Value.Expired,
                HttpContext.RequestAborted);
        }


        /// <summary>
        /// 去电转外部电话
        /// </summary>
        /// <param name="outerid"></param>
        /// <param name="outerphonenum"></param>
        /// <returns></returns>
        [HttpPost("OuterToOuter")]
        public async Task OuterToOuter(string outerid, string outerphonenum)
        {
            var result = await _client.OuterToOuter(new OuterToOuterRequest()
            {
                Attribute = "Connect",
                Outer = new List<OuterToOuterOuterModel>() { new OuterToOuterOuterModel() { Id = outerid }, new OuterToOuterOuterModel() { To = outerphonenum } },
            }, _options.Value.ReceiveKey,
                _options.Value.Expired,
                HttpContext.RequestAborted);
        }

        /// <summary>
        /// 去电转语音菜单
        /// </summary>
        /// <param name="outerid"></param>
        /// <param name="menuid"></param>
        /// <returns></returns>
        [HttpPost("OuterToMenu")]
        public async Task OuterToMenu(string outerid, string menuid)
        {
            var result = await _client.OuterToMenu(new OuterToMenuRequest()
            {
                Attribute = "Connect",
                Outer = new OuterToMenuOuter() { Id = outerid },
                Menu = new OuterToMenuMenu() { Id = menuid }
            }, _options.Value.ReceiveKey,
                _options.Value.Expired,
                HttpContext.RequestAborted);
        }

        /// <summary>
        /// 语音菜单呼分机
        /// </summary>
        /// <param name="menuid"></param>
        /// <param name="extid"></param>
        /// <returns></returns>
        [HttpPost("MenuToExt")]
        public async Task MenuToExt(string menuid, string extid)
        {
            var result = await _client.MenuToExt(new MenuToExtRequest()
            {
                Attribute = "Connect",
                Menu = new MenuToExtMenu() { Id = menuid },
                Ext = new MenuToExtExt() { Id = extid }
            }, _options.Value.ReceiveKey,
                _options.Value.Expired,
                HttpContext.RequestAborted);
        }

        /// <summary>
        /// 语音菜单呼外部电话
        /// </summary>
        /// <param name="menuid"></param>
        /// <param name="outernum"></param>
        /// <returns></returns>
        [HttpPost("MenuToOuter")]
        public async Task MenuToOuter(string menuid, string outernum)
        {
            var result = await _client.MenuToOuter(new MenuToOuterRequest()
            {
                Attribute = "Connect",
                Menu = new MenuToOuterMenu() { Id = menuid },
                Outer = new MenuToOuterOuter() { To = outernum }
            }, _options.Value.ReceiveKey,
                _options.Value.Expired,
                HttpContext.RequestAborted);
        }


        /// <summary>
        /// 双向呼叫（回拨）
        /// </summary>
        /// <param name="outerone"></param>
        /// <param name="outertwo"></param>
        /// <returns></returns>
        [HttpPost("TwoWayOuter")]
        public async Task TwoWayOuter(string outerone, string outertwo)
        {
            var result = await _client.TwoWayOuter(new TwoWayOuterRequest()
            {
                Attribute = "Connect",
                Outer = new List<TwoWayOuterOuter>()
                    {
                        new TwoWayOuterOuter(){ To = outerone},
                        new TwoWayOuterOuter(){ To = outertwo}
                    }
            }, _options.Value.ReceiveKey,
                _options.Value.Expired,
                HttpContext.RequestAborted);
        }

        /// <summary>
        /// 语音插播（分机）
        /// </summary>
        /// <param name="extid"></param>
        /// <param name="visitorid"></param>
        /// <param name="outerid"></param>
        /// <param name="voicefile"></param>
        /// <returns></returns>
        [HttpPost("VoiceNewsFlash")]
        public async Task VoiceNewsFlashExt(string voicefile, string extid)
        {
            var model = new VoiceNewsFlashRequest
            {
                Attribute = "Connect",
                VoiceFile = voicefile,
                Ext = new VoiceNewsFlashExt() { Id = extid }
            };

            var result = await _client.VoiceNewsFlash(model, _options.Value.ReceiveKey,
                _options.Value.Expired,
                HttpContext.RequestAborted);
        }

        /// <summary>
        /// 语音插播（来电）
        /// </summary>
        /// <param name="voicefile"></param>
        /// <param name="visiitorid"></param>
        /// <returns></returns>
        [HttpPost("VoiceNewsFlashVisitor")]
        public async Task VoiceNewsFlashVisitor(string voicefile, string visiitorid)
        {
            var model = new VoiceNewsFlashRequest
            {
                Attribute = "Connect",
                VoiceFile = voicefile,
                Visitor = new VoiceNewsFlashVisitor() { Id = visiitorid }
            };

            var result = await _client.VoiceNewsFlash(model, _options.Value.ReceiveKey,
                _options.Value.Expired,
                HttpContext.RequestAborted);
        }

        /// <summary>
        /// 语音插播（去电）
        /// </summary>
        /// <param name="voicefile"></param>
        /// <param name="outerid"></param>
        /// <returns></returns>
        [HttpPost("VoiceNewsFlashOuter")]
        public async Task VoiceNewsFlashOuter(string voicefile, string outerid)
        {
            var model = new VoiceNewsFlashRequest
            {
                Attribute = "Connect",
                VoiceFile = voicefile,
                Outer = new VoiceNewsFlashOuter() { Id = outerid }
            };

            var result = await _client.VoiceNewsFlash(model, _options.Value.ReceiveKey,
                _options.Value.Expired,
                HttpContext.RequestAborted);
        }


        #endregion


        #region 会议（Conference）

        /// <summary>
        /// 会议
        /// </summary>
        /// <param name="extid"></param>
        /// <returns></returns>
        [HttpPost("ConferenceMeeting")]
        public async Task ConferenceMeeting(string extid)
        {
            var result = await _client.ConferenceMeeting(new ConferenceMeetingRequest()
            {
                Attribute = "Conference",
                Ext = new ConferenceMeetingExt() { Id = extid }
            }, _options.Value.ReceiveKey,
                _options.Value.Expired,
                HttpContext.RequestAborted);
        }

        #endregion


        #region 队列（Queue）

        /// <summary>
        /// 来电转分机队列
        /// </summary>
        /// <param name="visitorid"></param>
        /// <param name="extid"></param>
        /// <returns></returns>
        [HttpPost("VisitorToExtQueue")]
        public async Task VisitorToExtQueue(string visitorid, string extid)
        {
            var result = await _client.VisitorToExtQueue(new VisitorToExtQueueRequest()
            {
                Attribute = "Queue",
                Visitor = new VisitorToExtQueueVisitor() { Id = visitorid },
                Ext = new VisitorToExtQueueExt() { Id = extid }
            }, _options.Value.ReceiveKey,
                _options.Value.Expired,
                HttpContext.RequestAborted);
        }

        /// <summary>
        ///来电转分机组队列
        /// </summary>
        /// <param name="visitorid"></param>
        /// <param name="groupid"></param>
        /// <returns></returns>
        [HttpPost("VisitorToGroupQueue")]
        public async Task VisitorToGroupQueue(string visitorid, string groupid)
        {
            var result = await _client.VisitorToGroupQueue(new VisitorToGroupQueueRequest()
            {
                Attribute = "Queue",
                Visitor = new VisitorToGroupQueueVisitor() { Id = visitorid },
                Group = new VisitorToGroupQueueGroup() { Id = groupid }
            },
                _options.Value.ReceiveKey,
                _options.Value.Expired,
                HttpContext.RequestAborted);
        }

        #endregion


        #endregion

        #region 语音管理命令

        /// <summary>
        /// 查询语音文件
        /// </summary>
        /// <returns></returns>
        [HttpGet("VoiceQueryList")]
        public async Task VoiceQueryList()
        {
            var result = await _client.VoiceQueryList(new VoiceQueryListRequest()
            {
                Attribute = "Query",
                VoiceFile = ""
            }, _options.Value.ReceiveKey,
                _options.Value.Expired,
                HttpContext.RequestAborted);
            _logger.LogInformation(System.Text.Json.JsonSerializer.Serialize(result));
        }

        /// <summary>
        /// 删除语音文件
        /// </summary>
        /// <param name="voiceFile"></param>
        /// <returns></returns>
        [HttpPost("RemoveVoiceFile")]
        public async Task RemoveVoiceFile(string voiceFile)
        {
            await _client.RemoveVoiceFile(new RemoveVoiceFileRequest()
            {
                Attribute = "Remove",
                VoiceFile = voiceFile
            }, _options.Value.ReceiveKey,
                _options.Value.Expired,
                HttpContext.RequestAborted);
        }

        #endregion

        #region 处理示忙的分机

        [HttpGet("configdisturb")]
        public async Task ConfigDisturb(string extid,string lineid)
        {
            var result = await _client.ConfigExt(new AssginConfigExtRequest()
            {
                Attribute = "Assign",
                Ext = new ConfigExt()
                {
                    Lineid = lineid,
                    Id = extid,
                   
                 
                    No_Disturb = "off",
                
                }
            },
            _options.Value.ReceiveKey,
            _options.Value.Expired,
            HttpContext.RequestAborted);
            _logger.LogInformation(System.Text.Json.JsonSerializer.Serialize(result));
        }


        #endregion

    }
}
